﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
  public sealed class GroupSymbol : Symbol, IReadOnlySymbol {
    internal List<ISymbol> symbols;

    [Storable]
    public IEnumerable<ISymbol> Symbols {
      get { return symbols; }
    }

    public override bool Enabled {
      get { return base.Enabled; }
      set {
        if (value != Enabled) {
          base.Enabled = value;
          symbols.ForEach(s => s.Enabled = value);
        }
      }
    }

    [StorableConstructor]
    private GroupSymbol(bool deserializing) : base(deserializing) { }
    private GroupSymbol(GroupSymbol original, Cloner cloner)
      : base(original, cloner) {
      symbols = original.Symbols.Select(s => cloner.Clone(s)).ToList();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GroupSymbol(this, cloner);
    }

    public GroupSymbol(string name, IEnumerable<ISymbol> symbols)
      : base(name, "A symbol which groups other symbols") {
      this.symbols = new List<ISymbol>(symbols);
      InitialFrequency = 0.0;
    }

    public IEnumerable<ISymbol> Flatten() {
      return symbols.Union(symbols.OfType<GroupSymbol>().SelectMany(g => g.Flatten()));
    }
  }
}
